from pydantic import SecretStr
from pydantic_settings import BaseSettings, SettingsConfigDict
from tpblite import CATEGORIES, ORDERS, TPB
from tpblite.models.torrents import Torrent as TpbTorrent, Torrents
import transmission_rpc


class Settings(BaseSettings):
    model_config = SettingsConfigDict(env_prefix="TD_", case_sensitive=False)

    tpb_site: str = "https://tpb.party"
    transmission_host: str = "192.168.0.29"
    transmission_port: int = 9091
    transmission_user: str = "transmission"
    transmission_pass: SecretStr = SecretStr("1304c2ea35e4e3f685691998c67ab58e20cf5d29CLC0a8Wn")


class TorrentDownloader:
    def __init__(self, config: Settings = Settings()):
        self.tpb = TPB(config.tpb_site)
        self.transmission = transmission_rpc.Client(
            host=config.transmission_host,
            port=config.transmission_port,
            username=config.transmission_user,
            password=config.transmission_pass.get_secret_value(),
        )

    def search(self, term: str, category: int = CATEGORIES.ALL) -> Torrents:
        return self.tpb.search(
            term,
            page=1,
            order=ORDERS.SEEDERS.DES,
            category=category,
        )

    def download(self, torrent: TpbTorrent):
        if torrent is not None and torrent.magnetlink:
            self.transmission.add_torrent(torrent.magnetlink)

    def download_magnet(self, magnetlink: str):
        self.transmission.add_torrent(magnetlink)

    def get_active_torrents(self) -> list[transmission_rpc.Torrent]:
        active, _ = self.transmission.get_recently_active_torrents()
        return active


def main():
    config = Settings()
    td = TorrentDownloader(config)

    initial = input("Search torrent: ")
    search_result: Torrents = td.search(initial, CATEGORIES.ALL)

    for idx, item in enumerate(search_result):
        print(f"{idx+1}. {item}")

    try:
        choice = int(input("Choose your destiny: "))
        if 30 > int(choice) > 0:
            torrent = search_result[choice - 1]
        else:
            print("Wrong destiny :( getting best torrent by hand")
            torrent = search_result.getBestTorrent(min_seeds=10)
    except Exception as _:
        torrent = search_result.getBestTorrent(min_seeds=10)

    if torrent is not None:
        td.download(torrent)
    else:
        print("Cannot find good enough torrent :D")
